
// Spawning tool.
TOOL.Category		= "VMF-Suite";
TOOL.Name		= "Super Select";
TOOL.Command		= nil;
TOOL.ConfigName		= "";


// Add language.
if ( CLIENT ) then
	language.Add( "select_tool", "Super Select" );	// ???
	language.Add( "Tool_superselect_tool_name", "VMF-Suite Super Select Tool" );
	language.Add( "Tool_superselect_tool_desc", "Generic VMF-Suite tool." );
	language.Add( "Tool_superselect_tool_0", "Left click to select an entity, Right click to deselect an entity." );
end

// Left click.
function TOOL:LeftClick( tr )
	// Don't allow the client to pass.
	if( CLIENT || SERVER ) then return true; end
	
	// didn't hit?
	if( !tr.HitNonWorld || !tr.Entity:IsValid() ) then return; end
	
	// grab list
	local list = {};
	self:SelectRadial( tr.Entity, list );
	for _, ent in pairs( list ) do
		VMFSuite:addEntityToSelection( self:GetOwner(), ent );
	end
	
	// process entity list.
	VMFSuite:wipePreview( self:GetOwner() );
	VMFSuite:doProcessEntityList( self:GetOwner() );

	return true;
end

// recursion.
function TOOL:SelectRadial( ent, list )
	// find all ents in range
	if( ent:IsValid() && list[ ent:EntIndex() ] ) then return; end
	list[ ent:EntIndex() ] = ent;
	
	// find in range
	local radius = ent:BoundingRadius() * 1.25;
	if( radius < 128 ) then radius = 128; end
	local ranged = ents.FindInSphere( ent:GetPos(), radius );
	local entity;
	for _, entity in pairs( ranged ) do
		if( entity:IsValid() && !entity:IsPlayer() && entity:GetSolid() != SOLID_NONE && entity:GetMoveType() != MOVETYPE_NONE ) then
			self:SelectRadial( entity, list );
			
		end
	end
	
end

// Right click.
function TOOL:RightClick( tr )
	// client be gone.. our newest product.
	if( CLIENT || SERVER ) then return true; end
	
	// wipe the preview away.
	VMFSuite:wipePreview( self:GetOwner() );
	
	// wipe the selection.
	VMFSuite:clearSelection( self:GetOwner() );
	
	// reset..
	self:GetOwner():GetTable().VMFSuite.ready_to_spawn = {};
	
	return true;
end

// Holster.
function TOOL:Holster( )
	// client be gone.. our newest product.
	if( CLIENT || SERVER ) then return; end
	
	// wipe the preview away.
	VMFSuite:wipePreview( self:GetOwner() );
	
	// reset..
	self:GetOwner():GetTable().VMFSuite.ready_to_spawn = {};
end

// reload
function TOOL:Reload( trace )
	// Don't allow the client to pass.
	if( CLIENT || SERVER ) then return true; end
	
	// Spawn.
	VMFSuite:loadReadyToSpawn( self:GetOwner(), trace.HitPos, false );
	
	return true;
end


// deploy.
function TOOL:Deploy( )
	// client be gone.. our newest product.
	if( CLIENT || SERVER ) then return; end

	// do we have a selection?
	local pl = self:GetOwner();
	VMFSuite:validateSelection( pl );
	if( table.getn( pl:GetTable().VMFSuite.selection ) > 0 ) then
		// process list!
		VMFSuite:doProcessEntityList( pl );
	end
end



// Build context panel.
function TOOL.BuildCPanel( CPanel )
	// header
	CPanel:AddControl( "Header", { Text = "#Tool_superselect_tool_name", Description	= "#Tool_superselect_tool_desc" }  )
	
	// presets.
	local params = { Label = "#Presets", MenuButton = 1, Folder = "vmf-suite", Options = {}, CVars = {} };
	params.Options.default = { vmf_rotation = 3, vmf_height = 0.1 };
	table.insert( params.CVars, "vmf_rotation" );
	table.insert( params.CVars, "vmf_height" );
	CPanel:AddControl( "ComboBox", params );

	
	// rotation
	CPanel:AddControl( "Slider",  { Label	= "Rotation",
					Type	= "Float",
					Min		= 0.0,
					Max		= 360.0,
					Command = "vmf_rotation" } );
			
	// height.		
	CPanel:AddControl( "Slider",  { Label	= "Height",
					Type	= "Integer",
					Min		= 0,
					Max		= 1024,
					Command = "vmf_height" } );
end
